/* Copyright (C) 2003 Univ. of Massachusetts Amherst, Computer Science Dept.
   This file is part of "MALLET" (MAchine Learning for LanguagE Toolkit).
   http://www.cs.umass.edu/~mccallum/mallet
   This software is provided under the terms of the Common Public License,
   version 1.0, as published by http://www.opensource.org.  For further
   information, see the file `LICENSE' included with this distribution. */

package cc.mallet.pipe.tests;


import static org.junit.Assert.assertEquals;

import java.io.ByteArrayOutputStream;
import java.io.PrintStream;

import org.junit.Test;

import cc.mallet.fst.tests.TestCRF;
import cc.mallet.pipe.CharSequence2TokenSequence;
import cc.mallet.pipe.Pipe;
import cc.mallet.pipe.PrintInputAndTarget;
import cc.mallet.pipe.SerialPipes;
import cc.mallet.pipe.TokenSequenceLowercase;
import cc.mallet.pipe.iterator.ArrayIterator;
import cc.mallet.pipe.tsf.OffsetConjunctions;
import cc.mallet.pipe.tsf.TokenText;
import cc.mallet.types.InstanceList;

/**
 *  Unit Test for class TestSpacePipe.java
 *
 *
 * Created: Thu Feb 26 14:56:55 2004
 *
 * @author <a href="mailto:casutton@cs.umass.edu">Charles Sutton</a>
 * @version $Id: TestSpacePipe.java,v 1.1 2007/10/22 21:37:40 mccallum Exp $
 */
public class TestSpacePipe {
    @Test
    public void testSpacePipe () {
        
        Pipe p = new SerialPipes(new Pipe[] {
            new CharSequence2TokenSequence("."),
            new TokenSequenceLowercase(),
            new TestCRF.TestCRFTokenSequenceRemoveSpaces (),
            new TokenText(),
            new OffsetConjunctions(false,
                                   new int[][] {{0},
                                                {1},{-1,0},{0,1},
                                                {-2,-1,0}, {0,1,2}, {-3,-2,-1}, {1,2,3},
                                   }),
            new PrintInputAndTarget(),
        });

        // Print to a string
        ByteArrayOutputStream out = new ByteArrayOutputStream ();
        PrintStream oldOut = System.out;
        System.setOut (new PrintStream (out));

        InstanceList lst = new InstanceList (p);
        lst.addThruPipe (new ArrayIterator(new String[] { TestCRF.data[0], TestCRF.data[1], }));
        System.setOut (oldOut);
        
        assertEquals (spacePipeOutput, out.toString().replaceAll("\\r\\n?", "\n"));
    }
        
    public static String spacePipeOutput =
        "name: array:0\ntarget: 0: start (0)\n1: notstart (1)\n2: notstart (1)\n3: notstart (1)\n4: start (0)\n5: notstart (1)\n6: notstart (1)\n7: notstart (1)\n8: notstart (1)\n9: notstart (1)\n10: notstart (1)\n11: notstart (1)\n12: start (0)\n13: notstart (1)\n14: start (0)\n15: start (0)\n16: notstart (1)\n17: notstart (1)\n18: notstart (1)\n19: notstart (1)\n20: notstart (1)\n21: start (0)\n22: notstart (1)\n23: start (0)\n24: notstart (1)\n25: notstart (1)\n26: start (0)\n27: notstart (1)\n28: notstart (1)\n29: notstart (1)\n30: notstart (1)\n31: notstart (1)\n32: start (0)\n33: notstart (1)\n34: notstart (1)\n35: notstart (1)\n36: notstart (1)\n37: notstart (1)\n38: notstart (1)\n39: start (0)\n40: notstart (1)\n41: start (0)\n42: notstart (1)\n43: notstart (1)\n44: notstart (1)\n45: start (0)\n46: notstart (1)\n47: notstart (1)\n48: notstart (1)\n49: notstart (1)\n50: start (0)\n51: notstart (1)\n52: notstart (1)\n53: notstart (1)\n54: notstart (1)\n55: notstart (1)\n56: notstart (1)\n57: notstart (1)\n58: notstart (1)\n59: notstart (1)\n60: notstart (1)\n61: start (0)\n62: notstart (1)\n63: notstart (1)\n64: notstart (1)\n65: notstart (1)\n66: notstart (1)\n67: start (0)\n68: notstart (1)\n69: notstart (1)\n70: notstart (1)\n71: notstart (1)\n72: notstart (1)\n73: start (0)\n74: notstart (1)\n75: notstart (1)\n76: start (0)\n77: notstart (1)\n78: notstart (1)\n79: notstart (1)\n80: notstart (1)\n81: notstart (1)\n82: notstart (1)\n83: start (0)\n84: notstart (1)\n85: notstart (1)\n86: start (0)\n87: notstart (1)\n88: notstart (1)\n89: notstart (1)\n90: notstart (1)\n91: notstart (1)\n92: notstart (1)\n93: notstart (1)\n94: notstart (1)\n95: start (0)\n96: notstart (1)\n97: notstart (1)\n98: notstart (1)\n99: start (0)\n100: notstart (1)\n101: notstart (1)\n102: notstart (1)\n103: notstart (1)\n104: notstart (1)\n105: notstart (1)\n106: notstart (1)\n107: notstart (1)\n108: notstart (1)\n109: start (0)\n110: notstart (1)\n111: start (0)\n112: notstart (1)\n113: notstart (1)\n114: notstart (1)\n115: notstart (1)\n116: notstart (1)\n117: start (0)\n118: notstart (1)\n119: start (0)\n120: notstart (1)\n121: notstart (1)\n122: notstart (1)\n123: start (0)\n124: notstart (1)\n125: notstart (1)\n126: notstart (1)\n127: notstart (1)\n128: start (0)\n129: notstart (1)\n130: start (0)\n131: notstart (1)\n132: notstart (1)\n133: notstart (1)\n134: notstart (1)\n135: notstart (1)\n136: notstart (1)\n137: notstart (1)\n138: start (0)\n139: notstart (1)\n140: notstart (1)\n141: start (0)\n142: notstart (1)\n143: notstart (1)\n144: start (0)\n145: notstart (1)\n146: notstart (1)\n147: notstart (1)\n148: notstart (1)\n149: start (0)\n150: notstart (1)\n151: start (0)\n152: notstart (1)\n153: notstart (1)\n154: start (0)\n155: notstart (1)\n156: notstart (1)\n157: notstart (1)\n158: notstart (1)\n159: notstart (1)\n160: notstart (1)\n161: notstart (1)\n162: notstart (1)\n\ninput: Token#0:f feature(r@1_&_e@2_&_e@3)=1.0 feature(<START2>@-3_&_<START1>@-2_&_<START0>@-1)=1.0 feature(f_&_r@1_&_e@2)=1.0 feature(<START1>@-2_&_<START0>@-1_&_f)=1.0 feature(f_&_r@1)=1.0 feature(<START0>@-1_&_f)=1.0 feature(r@1)=1.0 feature(f)=1.0  span[0..1]\nToken#1:r feature(e@1_&_e@2_&_s@3)=1.0 feature(<START1>@-3_&_<START0>@-2_&_f@-1)=1.0 feature(r_&_e@1_&_e@2)=1.0 feature(<START0>@-2_&_f@-1_&_r)=1.0 feature(r_&_e@1)=1.0 feature(f@-1_&_r)=1.0 feature(e@1)=1.0 feature(r)=1.0  span[1..2]\nToken#2:e feature(e@1_&_s@2_&_o@3)=1.0 feature(<START0>@-3_&_f@-2_&_r@-1)=1.0 feature(e_&_e@1_&_s@2)=1.0 feature(f@-2_&_r@-1_&_e)=1.0 feature(e_&_e@1)=1.0 feature(r@-1_&_e)=1.0 feature(e@1)=1.0 feature(e)=1.0  span[2..3]\nToken#3:e feature(s@1_&_o@2_&_f@3)=1.0 feature(f@-3_&_r@-2_&_e@-1)=1.0 feature(e_&_s@1_&_o@2)=1.0 feature(r@-2_&_e@-1_&_e)=1.0 feature(e_&_s@1)=1.0 feature(e@-1_&_e)=1.0 feature(s@1)=1.0 feature(e)=1.0  span[3..4]\nToken#4:s feature(o@1_&_f@2_&_t@3)=1.0 feature(r@-3_&_e@-2_&_e@-1)=1.0 feature(s_&_o@1_&_f@2)=1.0 feature(e@-2_&_e@-1_&_s)=1.0 feature(s_&_o@1)=1.0 feature(e@-1_&_s)=1.0 feature(o@1)=1.0 feature(s)=1.0  span[5..6]\nToken#5:o feature(f@1_&_t@2_&_w@3)=1.0 feature(e@-3_&_e@-2_&_s@-1)=1.0 feature(o_&_f@1_&_t@2)=1.0 feature(e@-2_&_s@-1_&_o)=1.0 feature(o_&_f@1)=1.0 feature(s@-1_&_o)=1.0 feature(f@1)=1.0 feature(o)=1.0  span[6..7]\nToken#6:f feature(t@1_&_w@2_&_a@3)=1.0 feature(e@-3_&_s@-2_&_o@-1)=1.0 feature(f_&_t@1_&_w@2)=1.0 feature(s@-2_&_o@-1_&_f)=1.0 feature(f_&_t@1)=1.0 feature(o@-1_&_f)=1.0 feature(t@1)=1.0 feature(f)=1.0  span[7..8]\nToken#7:t feature(w@1_&_a@2_&_r@3)=1.0 feature(s@-3_&_o@-2_&_f@-1)=1.0 feature(t_&_w@1_&_a@2)=1.0 feature(o@-2_&_f@-1_&_t)=1.0 feature(t_&_w@1)=1.0 feature(f@-1_&_t)=1.0 feature(w@1)=1.0 feature(t)=1.0  span[8..9]\nToken#8:w feature(a@1_&_r@2_&_e@3)=1.0 feature(o@-3_&_f@-2_&_t@-1)=1.0 feature(w_&_a@1_&_r@2)=1.0 feature(f@-2_&_t@-1_&_w)=1.0 feature(w_&_a@1)=1.0 feature(t@-1_&_w)=1.0 feature(a@1)=1.0 feature(w)=1.0  span[9..10]\nToken#9:a feature(r@1_&_e@2_&_i@3)=1.0 feature(f@-3_&_t@-2_&_w@-1)=1.0 feature(a_&_r@1_&_e@2)=1.0 feature(t@-2_&_w@-1_&_a)=1.0 feature(a_&_r@1)=1.0 feature(w@-1_&_a)=1.0 feature(r@1)=1.0 feature(a)=1.0  span[10..11]\nToken#10:r feature(e@1_&_i@2_&_s@3)=1.0 feature(t@-3_&_w@-2_&_a@-1)=1.0 feature(r_&_e@1_&_i@2)=1.0 feature(w@-2_&_a@-1_&_r)=1.0 feature(r_&_e@1)=1.0 feature(a@-1_&_r)=1.0 feature(e@1)=1.0 feature(r)=1.0  span[11..12]\nToken#11:e feature(i@1_&_s@2_&_a@3)=1.0 feature(w@-3_&_a@-2_&_r@-1)=1.0 feature(e_&_i@1_&_s@2)=1.0 feature(a@-2_&_r@-1_&_e)=1.0 feature(e_&_i@1)=1.0 feature(r@-1_&_e)=1.0 feature(i@1)=1.0 feature(e)=1.0  span[12..13]\nToken#12:i feature(s@1_&_a@2_&_m@3)=1.0 feature(a@-3_&_r@-2_&_e@-1)=1.0 feature(i_&_s@1_&_a@2)=1.0 feature(r@-2_&_e@-1_&_i)=1.0 feature(i_&_s@1)=1.0 feature(e@-1_&_i)=1.0 feature(s@1)=1.0 feature(i)=1.0  span[14..15]\nToken#13:s feature(a@1_&_m@2_&_a@3)=1.0 feature(r@-3_&_e@-2_&_i@-1)=1.0 feature(s_&_a@1_&_m@2)=1.0 feature(e@-2_&_i@-1_&_s)=1.0 feature(s_&_a@1)=1.0 feature(i@-1_&_s)=1.0 feature(a@1)=1.0 feature(s)=1.0  span[15..16]\nToken#14:a feature(m@1_&_a@2_&_t@3)=1.0 feature(e@-3_&_i@-2_&_s@-1)=1.0 feature(a_&_m@1_&_a@2)=1.0 feature(i@-2_&_s@-1_&_a)=1.0 feature(a_&_m@1)=1.0 feature(s@-1_&_a)=1.0 feature(m@1)=1.0 feature(a)=1.0  span[17..18]\nToken#15:m feature(a@1_&_t@2_&_t@3)=1.0 feature(i@-3_&_s@-2_&_a@-1)=1.0 feature(m_&_a@1_&_t@2)=1.0 feature(s@-2_&_a@-1_&_m)=1.0 feature(m_&_a@1)=1.0 feature(a@-1_&_m)=1.0 feature(a@1)=1.0 feature(m)=1.0  span[19..20]\nToken#16:a feature(t@1_&_t@2_&_e@3)=1.0 feature(s@-3_&_a@-2_&_m@-1)=1.0 feature(a_&_t@1_&_t@2)=1.0 feature(a@-2_&_m@-1_&_a)=1.0 feature(a_&_t@1)=1.0 feature(m@-1_&_a)=1.0 feature(t@1)=1.0 feature(a)=1.0  span[20..21]\nToken#17:t feature(t@1_&_e@2_&_r@3)=1.0 feature(a@-3_&_m@-2_&_a@-1)=1.0 feature(t_&_t@1_&_e@2)=1.0 feature(m@-2_&_a@-1_&_t)=1.0 feature(t_&_t@1)=1.0 feature(a@-1_&_t)=1.0 feature(t@1)=1.0 feature(t)=1.0  span[21..22]\nToken#18:t feature(e@1_&_r@2_&_o@3)=1.0 feature(m@-3_&_a@-2_&_t@-1)=1.0 feature(t_&_e@1_&_r@2)=1.0 feature(a@-2_&_t@-1_&_t)=1.0 feature(t_&_e@1)=1.0 feature(t@-1_&_t)=1.0 feature(e@1)=1.0 feature(t)=1.0  span[22..23]\nToken#19:e feature(r@1_&_o@2_&_f@3)=1.0 feature(a@-3_&_t@-2_&_t@-1)=1.0 feature(e_&_r@1_&_o@2)=1.0 feature(t@-2_&_t@-1_&_e)=1.0 feature(e_&_r@1)=1.0 feature(t@-1_&_e)=1.0 feature(r@1)=1.0 feature(e)=1.0  span[23..24]\nToken#20:r feature(o@1_&_f@2_&_t@3)=1.0 feature(t@-3_&_t@-2_&_e@-1)=1.0 feature(r_&_o@1_&_f@2)=1.0 feature(t@-2_&_e@-1_&_r)=1.0 feature(r_&_o@1)=1.0 feature(e@-1_&_r)=1.0 feature(o@1)=1.0 feature(r)=1.0  span[24..25]\nToken#21:o feature(f@1_&_t@2_&_h@3)=1.0 feature(t@-3_&_e@-2_&_r@-1)=1.0 feature(o_&_f@1_&_t@2)=1.0 feature(e@-2_&_r@-1_&_o)=1.0 feature(o_&_f@1)=1.0 feature(r@-1_&_o)=1.0 feature(f@1)=1.0 feature(o)=1.0  span[26..27]\nToken#22:f feature(t@1_&_h@2_&_e@3)=1.0 feature(e@-3_&_r@-2_&_o@-1)=1.0 feature(f_&_t@1_&_h@2)=1.0 feature(r@-2_&_o@-1_&_f)=1.0 feature(f_&_t@1)=1.0 feature(o@-1_&_f)=1.0 feature(t@1)=1.0 feature(f)=1.0  span[27..28]\nToken#23:t feature(h@1_&_e@2_&_u@3)=1.0 feature(r@-3_&_o@-2_&_f@-1)=1.0 feature(t_&_h@1_&_e@2)=1.0 feature(o@-2_&_f@-1_&_t)=1.0 feature(t_&_h@1)=1.0 feature(f@-1_&_t)=1.0 feature(h@1)=1.0 feature(t)=1.0  span[29..30]\nToken#24:h feature(e@1_&_u@2_&_s@3)=1.0 feature(o@-3_&_f@-2_&_t@-1)=1.0 feature(h_&_e@1_&_u@2)=1.0 feature(f@-2_&_t@-1_&_h)=1.0 feature(h_&_e@1)=1.0 feature(t@-1_&_h)=1.0 feature(e@1)=1.0 feature(h)=1.0  span[30..31]\nToken#25:e feature(u@1_&_s@2_&_e@3)=1.0 feature(f@-3_&_t@-2_&_h@-1)=1.0 feature(e_&_u@1_&_s@2)=1.0 feature(t@-2_&_h@-1_&_e)=1.0 feature(e_&_u@1)=1.0 feature(h@-1_&_e)=1.0 feature(u@1)=1.0 feature(e)=1.0  span[31..32]\nToken#26:u feature(s@1_&_e@2_&_r@3)=1.0 feature(t@-3_&_h@-2_&_e@-1)=1.0 feature(u_&_s@1_&_e@2)=1.0 feature(h@-2_&_e@-1_&_u)=1.0 feature(u_&_s@1)=1.0 feature(e@-1_&_u)=1.0 feature(s@1)=1.0 feature(u)=1.0  span[33..34]\nToken#27:s feature(e@1_&_r@2_&_s@3)=1.0 feature(h@-3_&_e@-2_&_u@-1)=1.0 feature(s_&_e@1_&_r@2)=1.0 feature(e@-2_&_u@-1_&_s)=1.0 feature(s_&_e@1)=1.0 feature(u@-1_&_s)=1.0 feature(e@1)=1.0 feature(s)=1.0  span[34..35]\nToken#28:e feature(r@1_&_s@2_&_'@3)=1.0 feature(e@-3_&_u@-2_&_s@-1)=1.0 feature(e_&_r@1_&_s@2)=1.0 feature(u@-2_&_s@-1_&_e)=1.0 feature(e_&_r@1)=1.0 feature(s@-1_&_e)=1.0 feature(r@1)=1.0 feature(e)=1.0  span[35..36]\nToken#29:r feature(s@1_&_'@2_&_f@3)=1.0 feature(u@-3_&_s@-2_&_e@-1)=1.0 feature(r_&_s@1_&_'@2)=1.0 feature(s@-2_&_e@-1_&_r)=1.0 feature(r_&_s@1)=1.0 feature(e@-1_&_r)=1.0 feature(s@1)=1.0 feature(r)=1.0  span[36..37]\nToken#30:s feature('@1_&_f@2_&_r@3)=1.0 feature(s@-3_&_e@-2_&_r@-1)=1.0 feature(s_&_'@1_&_f@2)=1.0 feature(e@-2_&_r@-1_&_s)=1.0 feature(s_&_'@1)=1.0 feature(r@-1_&_s)=1.0 feature('@1)=1.0 feature(s)=1.0  span[37..38]\nToken#31:' feature(f@1_&_r@2_&_e@3)=1.0 feature(e@-3_&_r@-2_&_s@-1)=1.0 feature('_&_f@1_&_r@2)=1.0 feature(r@-2_&_s@-1_&_')=1.0 feature('_&_f@1)=1.0 feature(s@-1_&_')=1.0 feature(f@1)=1.0 feature(')=1.0  span[38..39]\nToken#32:f feature(r@1_&_e@2_&_e@3)=1.0 feature(r@-3_&_s@-2_&_'@-1)=1.0 feature(f_&_r@1_&_e@2)=1.0 feature(s@-2_&_'@-1_&_f)=1.0 feature(f_&_r@1)=1.0 feature('@-1_&_f)=1.0 feature(r@1)=1.0 feature(f)=1.0  span[40..41]\nToken#33:r feature(e@1_&_e@2_&_d@3)=1.0 feature(s@-3_&_'@-2_&_f@-1)=1.0 feature(r_&_e@1_&_e@2)=1.0 feature('@-2_&_f@-1_&_r)=1.0 feature(r_&_e@1)=1.0 feature(f@-1_&_r)=1.0 feature(e@1)=1.0 feature(r)=1.0  span[41..42]\nToken#34:e feature(e@1_&_d@2_&_o@3)=1.0 feature('@-3_&_f@-2_&_r@-1)=1.0 feature(e_&_e@1_&_d@2)=1.0 feature(f@-2_&_r@-1_&_e)=1.0 feature(e_&_e@1)=1.0 feature(r@-1_&_e)=1.0 feature(e@1)=1.0 feature(e)=1.0  span[42..43]\nToken#35:e feature(d@1_&_o@2_&_m@3)=1.0 feature(f@-3_&_r@-2_&_e@-1)=1.0 feature(e_&_d@1_&_o@2)=1.0 feature(r@-2_&_e@-1_&_e)=1.0 feature(e_&_d@1)=1.0 feature(e@-1_&_e)=1.0 feature(d@1)=1.0 feature(e)=1.0  span[43..44]\nToken#36:d feature(o@1_&_m@2_&_t@3)=1.0 feature(r@-3_&_e@-2_&_e@-1)=1.0 feature(d_&_o@1_&_m@2)=1.0 feature(e@-2_&_e@-1_&_d)=1.0 feature(d_&_o@1)=1.0 feature(e@-1_&_d)=1.0 feature(o@1)=1.0 feature(d)=1.0  span[44..45]\nToken#37:o feature(m@1_&_t@2_&_o@3)=1.0 feature(e@-3_&_e@-2_&_d@-1)=1.0 feature(o_&_m@1_&_t@2)=1.0 feature(e@-2_&_d@-1_&_o)=1.0 feature(o_&_m@1)=1.0 feature(d@-1_&_o)=1.0 feature(m@1)=1.0 feature(o)=1.0  span[45..46]\nToken#38:m feature(t@1_&_o@2_&_r@3)=1.0 feature(e@-3_&_d@-2_&_o@-1)=1.0 feature(m_&_t@1_&_o@2)=1.0 feature(d@-2_&_o@-1_&_m)=1.0 feature(m_&_t@1)=1.0 feature(o@-1_&_m)=1.0 feature(t@1)=1.0 feature(m)=1.0  span[46..47]\nToken#39:t feature(o@1_&_r@2_&_u@3)=1.0 feature(d@-3_&_o@-2_&_m@-1)=1.0 feature(t_&_o@1_&_r@2)=1.0 feature(o@-2_&_m@-1_&_t)=1.0 feature(t_&_o@1)=1.0 feature(m@-1_&_t)=1.0 feature(o@1)=1.0 feature(t)=1.0  span[48..49]\nToken#40:o feature(r@1_&_u@2_&_n@3)=1.0 feature(o@-3_&_m@-2_&_t@-1)=1.0 feature(o_&_r@1_&_u@2)=1.0 feature(m@-2_&_t@-1_&_o)=1.0 feature(o_&_r@1)=1.0 feature(t@-1_&_o)=1.0 feature(r@1)=1.0 feature(o)=1.0  span[49..50]\nToken#41:r feature(u@1_&_n@2_&_,@3)=1.0 feature(m@-3_&_t@-2_&_o@-1)=1.0 feature(r_&_u@1_&_n@2)=1.0 feature(t@-2_&_o@-1_&_r)=1.0 feature(r_&_u@1)=1.0 feature(o@-1_&_r)=1.0 feature(u@1)=1.0 feature(r)=1.0  span[51..52]\nToken#42:u feature(n@1_&_,@2_&_c@3)=1.0 feature(t@-3_&_o@-2_&_r@-1)=1.0 feature(u_&_n@1_&_,@2)=1.0 feature(o@-2_&_r@-1_&_u)=1.0 feature(u_&_n@1)=1.0 feature(r@-1_&_u)=1.0 feature(n@1)=1.0 feature(u)=1.0  span[52..53]\nToken#43:n feature(,@1_&_c@2_&_o@3)=1.0 feature(o@-3_&_r@-2_&_u@-1)=1.0 feature(n_&_,@1_&_c@2)=1.0 feature(r@-2_&_u@-1_&_n)=1.0 feature(n_&_,@1)=1.0 feature(u@-1_&_n)=1.0 feature(,@1)=1.0 feature(n)=1.0  span[53..54]\nToken#44:, feature(c@1_&_o@2_&_p@3)=1.0 feature(r@-3_&_u@-2_&_n@-1)=1.0 feature(,_&_c@1_&_o@2)=1.0 feature(u@-2_&_n@-1_&_,)=1.0 feature(,_&_c@1)=1.0 feature(n@-1_&_,)=1.0 feature(c@1)=1.0 feature(,)=1.0  span[54..55]\nToken#45:c feature(o@1_&_p@2_&_y@3)=1.0 feature(u@-3_&_n@-2_&_,@-1)=1.0 feature(c_&_o@1_&_p@2)=1.0 feature(n@-2_&_,@-1_&_c)=1.0 feature(c_&_o@1)=1.0 feature(,@-1_&_c)=1.0 feature(o@1)=1.0 feature(c)=1.0  span[56..57]\nToken#46:o feature(p@1_&_y@2_&_,@3)=1.0 feature(n@-3_&_,@-2_&_c@-1)=1.0 feature(o_&_p@1_&_y@2)=1.0 feature(,@-2_&_c@-1_&_o)=1.0 feature(o_&_p@1)=1.0 feature(c@-1_&_o)=1.0 feature(p@1)=1.0 feature(o)=1.0  span[57..58]\nToken#47:p feature(y@1_&_,@2_&_d@3)=1.0 feature(,@-3_&_c@-2_&_o@-1)=1.0 feature(p_&_y@1_&_,@2)=1.0 feature(c@-2_&_o@-1_&_p)=1.0 feature(p_&_y@1)=1.0 feature(o@-1_&_p)=1.0 feature(y@1)=1.0 feature(p)=1.0  span[58..59]\nToken#48:y feature(,@1_&_d@2_&_i@3)=1.0 feature(c@-3_&_o@-2_&_p@-1)=1.0 feature(y_&_,@1_&_d@2)=1.0 feature(o@-2_&_p@-1_&_y)=1.0 feature(y_&_,@1)=1.0 feature(p@-1_&_y)=1.0 feature(,@1)=1.0 feature(y)=1.0  span[59..60]\nToken#49:, feature(d@1_&_i@2_&_s@3)=1.0 feature(o@-3_&_p@-2_&_y@-1)=1.0 feature(,_&_d@1_&_i@2)=1.0 feature(p@-2_&_y@-1_&_,)=1.0 feature(,_&_d@1)=1.0 feature(y@-1_&_,)=1.0 feature(d@1)=1.0 feature(,)=1.0  span[60..61]\nToken#50:d feature(i@1_&_s@2_&_t@3)=1.0 feature(p@-3_&_y@-2_&_,@-1)=1.0 feature(d_&_i@1_&_s@2)=1.0 feature(y@-2_&_,@-1_&_d)=1.0 feature(d_&_i@1)=1.0 feature(,@-1_&_d)=1.0 feature(i@1)=1.0 feature(d)=1.0  span[62..63]\nToken#51:i feature(s@1_&_t@2_&_r@3)=1.0 feature(y@-3_&_,@-2_&_d@-1)=1.0 feature(i_&_s@1_&_t@2)=1.0 feature(,@-2_&_d@-1_&_i)=1.0 feature(i_&_s@1)=1.0 feature(d@-1_&_i)=1.0 feature(s@1)=1.0 feature(i)=1.0  span[63..64]\nToken#52:s feature(t@1_&_r@2_&_i@3)=1.0 feature(,@-3_&_d@-2_&_i@-1)=1.0 feature(s_&_t@1_&_r@2)=1.0 feature(d@-2_&_i@-1_&_s)=1.0 feature(s_&_t@1)=1.0 feature(i@-1_&_s)=1.0 feature(t@1)=1.0 feature(s)=1.0  span[64..65]\nToken#53:t feature(r@1_&_i@2_&_b@3)=1.0 feature(d@-3_&_i@-2_&_s@-1)=1.0 feature(t_&_r@1_&_i@2)=1.0 feature(i@-2_&_s@-1_&_t)=1.0 feature(t_&_r@1)=1.0 feature(s@-1_&_t)=1.0 feature(r@1)=1.0 feature(t)=1.0  span[65..66]\nToken#54:r feature(i@1_&_b@2_&_u@3)=1.0 feature(i@-3_&_s@-2_&_t@-1)=1.0 feature(r_&_i@1_&_b@2)=1.0 feature(s@-2_&_t@-1_&_r)=1.0 feature(r_&_i@1)=1.0 feature(t@-1_&_r)=1.0 feature(i@1)=1.0 feature(r)=1.0  span[66..67]\nToken#55:i feature(b@1_&_u@2_&_t@3)=1.0 feature(s@-3_&_t@-2_&_r@-1)=1.0 feature(i_&_b@1_&_u@2)=1.0 feature(t@-2_&_r@-1_&_i)=1.0 feature(i_&_b@1)=1.0 feature(r@-1_&_i)=1.0 feature(b@1)=1.0 feature(i)=1.0  span[67..68]\nToken#56:b feature(u@1_&_t@2_&_e@3)=1.0 feature(t@-3_&_r@-2_&_i@-1)=1.0 feature(b_&_u@1_&_t@2)=1.0 feature(r@-2_&_i@-1_&_b)=1.0 feature(b_&_u@1)=1.0 feature(i@-1_&_b)=1.0 feature(u@1)=1.0 feature(b)=1.0  span[68..69]\nToken#57:u feature(t@1_&_e@2_&_,@3)=1.0 feature(r@-3_&_i@-2_&_b@-1)=1.0 feature(u_&_t@1_&_e@2)=1.0 feature(i@-2_&_b@-1_&_u)=1.0 feature(u_&_t@1)=1.0 feature(b@-1_&_u)=1.0 feature(t@1)=1.0 feature(u)=1.0  span[69..70]\nToken#58:t feature(e@1_&_,@2_&_s@3)=1.0 feature(i@-3_&_b@-2_&_u@-1)=1.0 feature(t_&_e@1_&_,@2)=1.0 feature(b@-2_&_u@-1_&_t)=1.0 feature(t_&_e@1)=1.0 feature(u@-1_&_t)=1.0 feature(e@1)=1.0 feature(t)=1.0  span[70..71]\nToken#59:e feature(,@1_&_s@2_&_t@3)=1.0 feature(b@-3_&_u@-2_&_t@-1)=1.0 feature(e_&_,@1_&_s@2)=1.0 feature(u@-2_&_t@-1_&_e)=1.0 feature(e_&_,@1)=1.0 feature(t@-1_&_e)=1.0 feature(,@1)=1.0 feature(e)=1.0  span[71..72]\nToken#60:, feature(s@1_&_t@2_&_u@3)=1.0 feature(u@-3_&_t@-2_&_e@-1)=1.0 feature(,_&_s@1_&_t@2)=1.0 feature(t@-2_&_e@-1_&_,)=1.0 feature(,_&_s@1)=1.0 feature(e@-1_&_,)=1.0 feature(s@1)=1.0 feature(,)=1.0  span[72..73]\nToken#61:s feature(t@1_&_u@2_&_d@3)=1.0 feature(t@-3_&_e@-2_&_,@-1)=1.0 feature(s_&_t@1_&_u@2)=1.0 feature(e@-2_&_,@-1_&_s)=1.0 feature(s_&_t@1)=1.0 feature(,@-1_&_s)=1.0 feature(t@1)=1.0 feature(s)=1.0  span[74..75]\nToken#62:t feature(u@1_&_d@2_&_y@3)=1.0 feature(e@-3_&_,@-2_&_s@-1)=1.0 feature(t_&_u@1_&_d@2)=1.0 feature(,@-2_&_s@-1_&_t)=1.0 feature(t_&_u@1)=1.0 feature(s@-1_&_t)=1.0 feature(u@1)=1.0 feature(t)=1.0  span[75..76]\nToken#63:u feature(d@1_&_y@2_&_,@3)=1.0 feature(,@-3_&_s@-2_&_t@-1)=1.0 feature(u_&_d@1_&_y@2)=1.0 feature(s@-2_&_t@-1_&_u)=1.0 feature(u_&_d@1)=1.0 feature(t@-1_&_u)=1.0 feature(d@1)=1.0 feature(u)=1.0  span[76..77]\nToken#64:d feature(y@1_&_,@2_&_c@3)=1.0 feature(s@-3_&_t@-2_&_u@-1)=1.0 feature(d_&_y@1_&_,@2)=1.0 feature(t@-2_&_u@-1_&_d)=1.0 feature(d_&_y@1)=1.0 feature(u@-1_&_d)=1.0 feature(y@1)=1.0 feature(d)=1.0  span[77..78]\nToken#65:y feature(,@1_&_c@2_&_h@3)=1.0 feature(t@-3_&_u@-2_&_d@-1)=1.0 feature(y_&_,@1_&_c@2)=1.0 feature(u@-2_&_d@-1_&_y)=1.0 feature(y_&_,@1)=1.0 feature(d@-1_&_y)=1.0 feature(,@1)=1.0 feature(y)=1.0  span[78..79]\nToken#66:, feature(c@1_&_h@2_&_a@3)=1.0 feature(u@-3_&_d@-2_&_y@-1)=1.0 feature(,_&_c@1_&_h@2)=1.0 feature(d@-2_&_y@-1_&_,)=1.0 feature(,_&_c@1)=1.0 feature(y@-1_&_,)=1.0 feature(c@1)=1.0 feature(,)=1.0  span[79..80]\nToken#67:c feature(h@1_&_a@2_&_n@3)=1.0 feature(d@-3_&_y@-2_&_,@-1)=1.0 feature(c_&_h@1_&_a@2)=1.0 feature(y@-2_&_,@-1_&_c)=1.0 feature(c_&_h@1)=1.0 feature(,@-1_&_c)=1.0 feature(h@1)=1.0 feature(c)=1.0  span[81..82]\nToken#68:h feature(a@1_&_n@2_&_g@3)=1.0 feature(y@-3_&_,@-2_&_c@-1)=1.0 feature(h_&_a@1_&_n@2)=1.0 feature(,@-2_&_c@-1_&_h)=1.0 feature(h_&_a@1)=1.0 feature(c@-1_&_h)=1.0 feature(a@1)=1.0 feature(h)=1.0  span[82..83]\nToken#69:a feature(n@1_&_g@2_&_e@3)=1.0 feature(,@-3_&_c@-2_&_h@-1)=1.0 feature(a_&_n@1_&_g@2)=1.0 feature(c@-2_&_h@-1_&_a)=1.0 feature(a_&_n@1)=1.0 feature(h@-1_&_a)=1.0 feature(n@1)=1.0 feature(a)=1.0  span[83..84]\nToken#70:n feature(g@1_&_e@2_&_a@3)=1.0 feature(c@-3_&_h@-2_&_a@-1)=1.0 feature(n_&_g@1_&_e@2)=1.0 feature(h@-2_&_a@-1_&_n)=1.0 feature(n_&_g@1)=1.0 feature(a@-1_&_n)=1.0 feature(g@1)=1.0 feature(n)=1.0  span[84..85]\nToken#71:g feature(e@1_&_a@2_&_n@3)=1.0 feature(h@-3_&_a@-2_&_n@-1)=1.0 feature(g_&_e@1_&_a@2)=1.0 feature(a@-2_&_n@-1_&_g)=1.0 feature(g_&_e@1)=1.0 feature(n@-1_&_g)=1.0 feature(e@1)=1.0 feature(g)=1.0  span[85..86]\nToken#72:e feature(a@1_&_n@2_&_d@3)=1.0 feature(a@-3_&_n@-2_&_g@-1)=1.0 feature(e_&_a@1_&_n@2)=1.0 feature(n@-2_&_g@-1_&_e)=1.0 feature(e_&_a@1)=1.0 feature(g@-1_&_e)=1.0 feature(a@1)=1.0 feature(e)=1.0  span[86..87]\nToken#73:a feature(n@1_&_d@2_&_i@3)=1.0 feature(n@-3_&_g@-2_&_e@-1)=1.0 feature(a_&_n@1_&_d@2)=1.0 feature(g@-2_&_e@-1_&_a)=1.0 feature(a_&_n@1)=1.0 feature(e@-1_&_a)=1.0 feature(n@1)=1.0 feature(a)=1.0  span[88..89]\nToken#74:n feature(d@1_&_i@2_&_m@3)=1.0 feature(g@-3_&_e@-2_&_a@-1)=1.0 feature(n_&_d@1_&_i@2)=1.0 feature(e@-2_&_a@-1_&_n)=1.0 feature(n_&_d@1)=1.0 feature(a@-1_&_n)=1.0 feature(d@1)=1.0 feature(n)=1.0  span[89..90]\nToken#75:d feature(i@1_&_m@2_&_p@3)=1.0 feature(e@-3_&_a@-2_&_n@-1)=1.0 feature(d_&_i@1_&_m@2)=1.0 feature(a@-2_&_n@-1_&_d)=1.0 feature(d_&_i@1)=1.0 feature(n@-1_&_d)=1.0 feature(i@1)=1.0 feature(d)=1.0  span[90..91]\nToken#76:i feature(m@1_&_p@2_&_r@3)=1.0 feature(a@-3_&_n@-2_&_d@-1)=1.0 feature(i_&_m@1_&_p@2)=1.0 feature(n@-2_&_d@-1_&_i)=1.0 feature(i_&_m@1)=1.0 feature(d@-1_&_i)=1.0 feature(m@1)=1.0 feature(i)=1.0  span[92..93]\nToken#77:m feature(p@1_&_r@2_&_o@3)=1.0 feature(n@-3_&_d@-2_&_i@-1)=1.0 feature(m_&_p@1_&_r@2)=1.0 feature(d@-2_&_i@-1_&_m)=1.0 feature(m_&_p@1)=1.0 feature(i@-1_&_m)=1.0 feature(p@1)=1.0 feature(m)=1.0  span[93..94]\nToken#78:p feature(r@1_&_o@2_&_v@3)=1.0 feature(d@-3_&_i@-2_&_m@-1)=1.0 feature(p_&_r@1_&_o@2)=1.0 feature(i@-2_&_m@-1_&_p)=1.0 feature(p_&_r@1)=1.0 feature(m@-1_&_p)=1.0 feature(r@1)=1.0 feature(p)=1.0  span[94..95]\nToken#79:r feature(o@1_&_v@2_&_e@3)=1.0 feature(i@-3_&_m@-2_&_p@-1)=1.0 feature(r_&_o@1_&_v@2)=1.0 feature(m@-2_&_p@-1_&_r)=1.0 feature(r_&_o@1)=1.0 feature(p@-1_&_r)=1.0 feature(o@1)=1.0 feature(r)=1.0  span[95..96]\nToken#80:o feature(v@1_&_e@2_&_t@3)=1.0 feature(m@-3_&_p@-2_&_r@-1)=1.0 feature(o_&_v@1_&_e@2)=1.0 feature(p@-2_&_r@-1_&_o)=1.0 feature(o_&_v@1)=1.0 feature(r@-1_&_o)=1.0 feature(v@1)=1.0 feature(o)=1.0  span[96..97]\nToken#81:v feature(e@1_&_t@2_&_h@3)=1.0 feature(p@-3_&_r@-2_&_o@-1)=1.0 feature(v_&_e@1_&_t@2)=1.0 feature(r@-2_&_o@-1_&_v)=1.0 feature(v_&_e@1)=1.0 feature(o@-1_&_v)=1.0 feature(e@1)=1.0 feature(v)=1.0  span[97..98]\nToken#82:e feature(t@1_&_h@2_&_e@3)=1.0 feature(r@-3_&_o@-2_&_v@-1)=1.0 feature(e_&_t@1_&_h@2)=1.0 feature(o@-2_&_v@-1_&_e)=1.0 feature(e_&_t@1)=1.0 feature(v@-1_&_e)=1.0 feature(t@1)=1.0 feature(e)=1.0  span[98..99]\nToken#83:t feature(h@1_&_e@2_&_s@3)=1.0 feature(o@-3_&_v@-2_&_e@-1)=1.0 feature(t_&_h@1_&_e@2)=1.0 feature(v@-2_&_e@-1_&_t)=1.0 feature(t_&_h@1)=1.0 feature(e@-1_&_t)=1.0 feature(h@1)=1.0 feature(t)=1.0  span[100..101]\nToken#84:h feature(e@1_&_s@2_&_o@3)=1.0 feature(v@-3_&_e@-2_&_t@-1)=1.0 feature(h_&_e@1_&_s@2)=1.0 feature(e@-2_&_t@-1_&_h)=1.0 feature(h_&_e@1)=1.0 feature(t@-1_&_h)=1.0 feature(e@1)=1.0 feature(h)=1.0  span[101..102]\nToken#85:e feature(s@1_&_o@2_&_f@3)=1.0 feature(e@-3_&_t@-2_&_h@-1)=1.0 feature(e_&_s@1_&_o@2)=1.0 feature(t@-2_&_h@-1_&_e)=1.0 feature(e_&_s@1)=1.0 feature(h@-1_&_e)=1.0 feature(s@1)=1.0 feature(e)=1.0  span[102..103]\nToken#86:s feature(o@1_&_f@2_&_t@3)=1.0 feature(t@-3_&_h@-2_&_e@-1)=1.0 feature(s_&_o@1_&_f@2)=1.0 feature(h@-2_&_e@-1_&_s)=1.0 feature(s_&_o@1)=1.0 feature(e@-1_&_s)=1.0 feature(o@1)=1.0 feature(s)=1.0  span[104..105]\nToken#87:o feature(f@1_&_t@2_&_w@3)=1.0 feature(h@-3_&_e@-2_&_s@-1)=1.0 feature(o_&_f@1_&_t@2)=1.0 feature(e@-2_&_s@-1_&_o)=1.0 feature(o_&_f@1)=1.0 feature(s@-1_&_o)=1.0 feature(f@1)=1.0 feature(o)=1.0  span[105..106]\nToken#88:f feature(t@1_&_w@2_&_a@3)=1.0 feature(e@-3_&_s@-2_&_o@-1)=1.0 feature(f_&_t@1_&_w@2)=1.0 feature(s@-2_&_o@-1_&_f)=1.0 feature(f_&_t@1)=1.0 feature(o@-1_&_f)=1.0 feature(t@1)=1.0 feature(f)=1.0  span[106..107]\nToken#89:t feature(w@1_&_a@2_&_r@3)=1.0 feature(s@-3_&_o@-2_&_f@-1)=1.0 feature(t_&_w@1_&_a@2)=1.0 feature(o@-2_&_f@-1_&_t)=1.0 feature(t_&_w@1)=1.0 feature(f@-1_&_t)=1.0 feature(w@1)=1.0 feature(t)=1.0  span[107..108]\nToken#90:w feature(a@1_&_r@2_&_e@3)=1.0 feature(o@-3_&_f@-2_&_t@-1)=1.0 feature(w_&_a@1_&_r@2)=1.0 feature(f@-2_&_t@-1_&_w)=1.0 feature(w_&_a@1)=1.0 feature(t@-1_&_w)=1.0 feature(a@1)=1.0 feature(w)=1.0  span[108..109]\nToken#91:a feature(r@1_&_e@2_&_.@3)=1.0 feature(f@-3_&_t@-2_&_w@-1)=1.0 feature(a_&_r@1_&_e@2)=1.0 feature(t@-2_&_w@-1_&_a)=1.0 feature(a_&_r@1)=1.0 feature(w@-1_&_a)=1.0 feature(r@1)=1.0 feature(a)=1.0  span[109..110]\nToken#92:r feature(e@1_&_.@2_&_m@3)=1.0 feature(t@-3_&_w@-2_&_a@-1)=1.0 feature(r_&_e@1_&_.@2)=1.0 feature(w@-2_&_a@-1_&_r)=1.0 feature(r_&_e@1)=1.0 feature(a@-1_&_r)=1.0 feature(e@1)=1.0 feature(r)=1.0  span[110..111]\nToken#93:e feature(.@1_&_m@2_&_o@3)=1.0 feature(w@-3_&_a@-2_&_r@-1)=1.0 feature(e_&_.@1_&_m@2)=1.0 feature(a@-2_&_r@-1_&_e)=1.0 feature(e_&_.@1)=1.0 feature(r@-1_&_e)=1.0 feature(.@1)=1.0 feature(e)=1.0  span[111..112]\nToken#94:. feature(m@1_&_o@2_&_r@3)=1.0 feature(a@-3_&_r@-2_&_e@-1)=1.0 feature(._&_m@1_&_o@2)=1.0 feature(r@-2_&_e@-1_&_.)=1.0 feature(._&_m@1)=1.0 feature(e@-1_&_.)=1.0 feature(m@1)=1.0 feature(.)=1.0  span[112..113]\nToken#95:m feature(o@1_&_r@2_&_e@3)=1.0 feature(r@-3_&_e@-2_&_.@-1)=1.0 feature(m_&_o@1_&_r@2)=1.0 feature(e@-2_&_.@-1_&_m)=1.0 feature(m_&_o@1)=1.0 feature(.@-1_&_m)=1.0 feature(o@1)=1.0 feature(m)=1.0  span[114..115]\nToken#96:o feature(r@1_&_e@2_&_p@3)=1.0 feature(e@-3_&_.@-2_&_m@-1)=1.0 feature(o_&_r@1_&_e@2)=1.0 feature(.@-2_&_m@-1_&_o)=1.0 feature(o_&_r@1)=1.0 feature(m@-1_&_o)=1.0 feature(r@1)=1.0 feature(o)=1.0  span[115..116]\nToken#97:r feature(e@1_&_p@2_&_r@3)=1.0 feature(.@-3_&_m@-2_&_o@-1)=1.0 feature(r_&_e@1_&_p@2)=1.0 feature(m@-2_&_o@-1_&_r)=1.0 feature(r_&_e@1)=1.0 feature(o@-1_&_r)=1.0 feature(e@1)=1.0 feature(r)=1.0  span[116..117]\nToken#98:e feature(p@1_&_r@2_&_e@3)=1.0 feature(m@-3_&_o@-2_&_r@-1)=1.0 feature(e_&_p@1_&_r@2)=1.0 feature(o@-2_&_r@-1_&_e)=1.0 feature(e_&_p@1)=1.0 feature(r@-1_&_e)=1.0 feature(p@1)=1.0 feature(e)=1.0  span[117..118]\nToken#99:p feature(r@1_&_e@2_&_c@3)=1.0 feature(o@-3_&_r@-2_&_e@-1)=1.0 feature(p_&_r@1_&_e@2)=1.0 feature(r@-2_&_e@-1_&_p)=1.0 feature(p_&_r@1)=1.0 feature(e@-1_&_p)=1.0 feature(r@1)=1.0 feature(p)=1.0  span[119..120]\nToken#100:r feature(e@1_&_c@2_&_i@3)=1.0 feature(r@-3_&_e@-2_&_p@-1)=1.0 feature(r_&_e@1_&_c@2)=1.0 feature(e@-2_&_p@-1_&_r)=1.0 feature(r_&_e@1)=1.0 feature(p@-1_&_r)=1.0 feature(e@1)=1.0 feature(r)=1.0  span[120..121]\nToken#101:e feature(c@1_&_i@2_&_s@3)=1.0 feature(e@-3_&_p@-2_&_r@-1)=1.0 feature(e_&_c@1_&_i@2)=1.0 feature(p@-2_&_r@-1_&_e)=1.0 feature(e_&_c@1)=1.0 feature(r@-1_&_e)=1.0 feature(c@1)=1.0 feature(e)=1.0  span[121..122]\nToken#102:c feature(i@1_&_s@2_&_e@3)=1.0 feature(p@-3_&_r@-2_&_e@-1)=1.0 feature(c_&_i@1_&_s@2)=1.0 feature(r@-2_&_e@-1_&_c)=1.0 feature(c_&_i@1)=1.0 feature(e@-1_&_c)=1.0 feature(i@1)=1.0 feature(c)=1.0  span[122..123]\nToken#103:i feature(s@1_&_e@2_&_l@3)=1.0 feature(r@-3_&_e@-2_&_c@-1)=1.0 feature(i_&_s@1_&_e@2)=1.0 feature(e@-2_&_c@-1_&_i)=1.0 feature(i_&_s@1)=1.0 feature(c@-1_&_i)=1.0 feature(s@1)=1.0 feature(i)=1.0  span[123..124]\nToken#104:s feature(e@1_&_l@2_&_y@3)=1.0 feature(e@-3_&_c@-2_&_i@-1)=1.0 feature(s_&_e@1_&_l@2)=1.0 feature(c@-2_&_i@-1_&_s)=1.0 feature(s_&_e@1)=1.0 feature(i@-1_&_s)=1.0 feature(e@1)=1.0 feature(s)=1.0  span[124..125]\nToken#105:e feature(l@1_&_y@2_&_,@3)=1.0 feature(c@-3_&_i@-2_&_s@-1)=1.0 feature(e_&_l@1_&_y@2)=1.0 feature(i@-2_&_s@-1_&_e)=1.0 feature(e_&_l@1)=1.0 feature(s@-1_&_e)=1.0 feature(l@1)=1.0 feature(e)=1.0  span[125..126]\nToken#106:l feature(y@1_&_,@2_&_i@3)=1.0 feature(i@-3_&_s@-2_&_e@-1)=1.0 feature(l_&_y@1_&_,@2)=1.0 feature(s@-2_&_e@-1_&_l)=1.0 feature(l_&_y@1)=1.0 feature(e@-1_&_l)=1.0 feature(y@1)=1.0 feature(l)=1.0  span[126..127]\nToken#107:y feature(,@1_&_i@2_&_t@3)=1.0 feature(s@-3_&_e@-2_&_l@-1)=1.0 feature(y_&_,@1_&_i@2)=1.0 feature(e@-2_&_l@-1_&_y)=1.0 feature(y_&_,@1)=1.0 feature(l@-1_&_y)=1.0 feature(,@1)=1.0 feature(y)=1.0  span[127..128]\nToken#108:, feature(i@1_&_t@2_&_r@3)=1.0 feature(e@-3_&_l@-2_&_y@-1)=1.0 feature(,_&_i@1_&_t@2)=1.0 feature(l@-2_&_y@-1_&_,)=1.0 feature(,_&_i@1)=1.0 feature(y@-1_&_,)=1.0 feature(i@1)=1.0 feature(,)=1.0  span[128..129]\nToken#109:i feature(t@1_&_r@2_&_e@3)=1.0 feature(l@-3_&_y@-2_&_,@-1)=1.0 feature(i_&_t@1_&_r@2)=1.0 feature(y@-2_&_,@-1_&_i)=1.0 feature(i_&_t@1)=1.0 feature(,@-1_&_i)=1.0 feature(t@1)=1.0 feature(i)=1.0  span[130..131]\nToken#110:t feature(r@1_&_e@2_&_f@3)=1.0 feature(y@-3_&_,@-2_&_i@-1)=1.0 feature(t_&_r@1_&_e@2)=1.0 feature(,@-2_&_i@-1_&_t)=1.0 feature(t_&_r@1)=1.0 feature(i@-1_&_t)=1.0 feature(r@1)=1.0 feature(t)=1.0  span[131..132]\nToken#111:r feature(e@1_&_f@2_&_e@3)=1.0 feature(,@-3_&_i@-2_&_t@-1)=1.0 feature(r_&_e@1_&_f@2)=1.0 feature(i@-2_&_t@-1_&_r)=1.0 feature(r_&_e@1)=1.0 feature(t@-1_&_r)=1.0 feature(e@1)=1.0 feature(r)=1.0  span[133..134]\nToken#112:e feature(f@1_&_e@2_&_r@3)=1.0 feature(i@-3_&_t@-2_&_r@-1)=1.0 feature(e_&_f@1_&_e@2)=1.0 feature(t@-2_&_r@-1_&_e)=1.0 feature(e_&_f@1)=1.0 feature(r@-1_&_e)=1.0 feature(f@1)=1.0 feature(e)=1.0  span[134..135]\nToken#113:f feature(e@1_&_r@2_&_s@3)=1.0 feature(t@-3_&_r@-2_&_e@-1)=1.0 feature(f_&_e@1_&_r@2)=1.0 feature(r@-2_&_e@-1_&_f)=1.0 feature(f_&_e@1)=1.0 feature(e@-1_&_f)=1.0 feature(e@1)=1.0 feature(f)=1.0  span[135..136]\nToken#114:e feature(r@1_&_s@2_&_t@3)=1.0 feature(r@-3_&_e@-2_&_f@-1)=1.0 feature(e_&_r@1_&_s@2)=1.0 feature(e@-2_&_f@-1_&_e)=1.0 feature(e_&_r@1)=1.0 feature(f@-1_&_e)=1.0 feature(r@1)=1.0 feature(e)=1.0  span[136..137]\nToken#115:r feature(s@1_&_t@2_&_o@3)=1.0 feature(e@-3_&_f@-2_&_e@-1)=1.0 feature(r_&_s@1_&_t@2)=1.0 feature(f@-2_&_e@-1_&_r)=1.0 feature(r_&_s@1)=1.0 feature(e@-1_&_r)=1.0 feature(s@1)=1.0 feature(r)=1.0  span[137..138]\nToken#116:s feature(t@1_&_o@2_&_f@3)=1.0 feature(f@-3_&_e@-2_&_r@-1)=1.0 feature(s_&_t@1_&_o@2)=1.0 feature(e@-2_&_r@-1_&_s)=1.0 feature(s_&_t@1)=1.0 feature(r@-1_&_s)=1.0 feature(t@1)=1.0 feature(s)=1.0  span[138..139]\nToken#117:t feature(o@1_&_f@2_&_o@3)=1.0 feature(e@-3_&_r@-2_&_s@-1)=1.0 feature(t_&_o@1_&_f@2)=1.0 feature(r@-2_&_s@-1_&_t)=1.0 feature(t_&_o@1)=1.0 feature(s@-1_&_t)=1.0 feature(o@1)=1.0 feature(t)=1.0  span[140..141]\nToken#118:o feature(f@1_&_o@2_&_u@3)=1.0 feature(r@-3_&_s@-2_&_t@-1)=1.0 feature(o_&_f@1_&_o@2)=1.0 feature(s@-2_&_t@-1_&_o)=1.0 feature(o_&_f@1)=1.0 feature(t@-1_&_o)=1.0 feature(f@1)=1.0 feature(o)=1.0  span[141..142]\nToken#119:f feature(o@1_&_u@2_&_r@3)=1.0 feature(s@-3_&_t@-2_&_o@-1)=1.0 feature(f_&_o@1_&_u@2)=1.0 feature(t@-2_&_o@-1_&_f)=1.0 feature(f_&_o@1)=1.0 feature(o@-1_&_f)=1.0 feature(o@1)=1.0 feature(f)=1.0  span[143..144]\nToken#120:o feature(u@1_&_r@2_&_k@3)=1.0 feature(t@-3_&_o@-2_&_f@-1)=1.0 feature(o_&_u@1_&_r@2)=1.0 feature(o@-2_&_f@-1_&_o)=1.0 feature(o_&_u@1)=1.0 feature(f@-1_&_o)=1.0 feature(u@1)=1.0 feature(o)=1.0  span[144..145]\nToken#121:u feature(r@1_&_k@2_&_i@3)=1.0 feature(o@-3_&_f@-2_&_o@-1)=1.0 feature(u_&_r@1_&_k@2)=1.0 feature(f@-2_&_o@-1_&_u)=1.0 feature(u_&_r@1)=1.0 feature(o@-1_&_u)=1.0 feature(r@1)=1.0 feature(u)=1.0  span[145..146]\nToken#122:r feature(k@1_&_i@2_&_n@3)=1.0 feature(f@-3_&_o@-2_&_u@-1)=1.0 feature(r_&_k@1_&_i@2)=1.0 feature(o@-2_&_u@-1_&_r)=1.0 feature(r_&_k@1)=1.0 feature(u@-1_&_r)=1.0 feature(k@1)=1.0 feature(r)=1.0  span[146..147]\nToken#123:k feature(i@1_&_n@2_&_d@3)=1.0 feature(o@-3_&_u@-2_&_r@-1)=1.0 feature(k_&_i@1_&_n@2)=1.0 feature(u@-2_&_r@-1_&_k)=1.0 feature(k_&_i@1)=1.0 feature(r@-1_&_k)=1.0 feature(i@1)=1.0 feature(k)=1.0  span[148..149]\nToken#124:i feature(n@1_&_d@2_&_s@3)=1.0 feature(u@-3_&_r@-2_&_k@-1)=1.0 feature(i_&_n@1_&_d@2)=1.0 feature(r@-2_&_k@-1_&_i)=1.0 feature(i_&_n@1)=1.0 feature(k@-1_&_i)=1.0 feature(n@1)=1.0 feature(i)=1.0  span[149..150]\nToken#125:n feature(d@1_&_s@2_&_o@3)=1.0 feature(r@-3_&_k@-2_&_i@-1)=1.0 feature(n_&_d@1_&_s@2)=1.0 feature(k@-2_&_i@-1_&_n)=1.0 feature(n_&_d@1)=1.0 feature(i@-1_&_n)=1.0 feature(d@1)=1.0 feature(n)=1.0  span[150..151]\nToken#126:d feature(s@1_&_o@2_&_f@3)=1.0 feature(k@-3_&_i@-2_&_n@-1)=1.0 feature(d_&_s@1_&_o@2)=1.0 feature(i@-2_&_n@-1_&_d)=1.0 feature(d_&_s@1)=1.0 feature(n@-1_&_d)=1.0 feature(s@1)=1.0 feature(d)=1.0  span[151..152]\nToken#127:s feature(o@1_&_f@2_&_f@3)=1.0 feature(i@-3_&_n@-2_&_d@-1)=1.0 feature(s_&_o@1_&_f@2)=1.0 feature(n@-2_&_d@-1_&_s)=1.0 feature(s_&_o@1)=1.0 feature(d@-1_&_s)=1.0 feature(o@1)=1.0 feature(s)=1.0  span[152..153]\nToken#128:o feature(f@1_&_f@2_&_r@3)=1.0 feature(n@-3_&_d@-2_&_s@-1)=1.0 feature(o_&_f@1_&_f@2)=1.0 feature(d@-2_&_s@-1_&_o)=1.0 feature(o_&_f@1)=1.0 feature(s@-1_&_o)=1.0 feature(f@1)=1.0 feature(o)=1.0  span[154..155]\nToken#129:f feature(f@1_&_r@2_&_e@3)=1.0 feature(d@-3_&_s@-2_&_o@-1)=1.0 feature(f_&_f@1_&_r@2)=1.0 feature(s@-2_&_o@-1_&_f)=1.0 feature(f_&_f@1)=1.0 feature(o@-1_&_f)=1.0 feature(f@1)=1.0 feature(f)=1.0  span[155..156]\nToken#130:f feature(r@1_&_e@2_&_e@3)=1.0 feature(s@-3_&_o@-2_&_f@-1)=1.0 feature(f_&_r@1_&_e@2)=1.0 feature(o@-2_&_f@-1_&_f)=1.0 feature(f_&_r@1)=1.0 feature(f@-1_&_f)=1.0 feature(r@1)=1.0 feature(f)=1.0  span[157..158]\nToken#131:r feature(e@1_&_e@2_&_d@3)=1.0 feature(o@-3_&_f@-2_&_f@-1)=1.0 feature(r_&_e@1_&_e@2)=1.0 feature(f@-2_&_f@-1_&_r)=1.0 feature(r_&_e@1)=1.0 feature(f@-1_&_r)=1.0 feature(e@1)=1.0 feature(r)=1.0  span[158..159]\nToken#132:e feature(e@1_&_d@2_&_o@3)=1.0 feature(f@-3_&_f@-2_&_r@-1)=1.0 feature(e_&_e@1_&_d@2)=1.0 feature(f@-2_&_r@-1_&_e)=1.0 feature(e_&_e@1)=1.0 feature(r@-1_&_e)=1.0 feature(e@1)=1.0 feature(e)=1.0  span[159..160]\nToken#133:e feature(d@1_&_o@2_&_m@3)=1.0 feature(f@-3_&_r@-2_&_e@-1)=1.0 feature(e_&_d@1_&_o@2)=1.0 feature(r@-2_&_e@-1_&_e)=1.0 feature(e_&_d@1)=1.0 feature(e@-1_&_e)=1.0 feature(d@1)=1.0 feature(e)=1.0  span[160..161]\nToken#134:d feature(o@1_&_m@2_&_,@3)=1.0 feature(r@-3_&_e@-2_&_e@-1)=1.0 feature(d_&_o@1_&_m@2)=1.0 feature(e@-2_&_e@-1_&_d)=1.0 feature(d_&_o@1)=1.0 feature(e@-1_&_d)=1.0 feature(o@1)=1.0 feature(d)=1.0  span[161..162]\nToken#135:o feature(m@1_&_,@2_&_f@3)=1.0 feature(e@-3_&_e@-2_&_d@-1)=1.0 feature(o_&_m@1_&_,@2)=1.0 feature(e@-2_&_d@-1_&_o)=1.0 feature(o_&_m@1)=1.0 feature(d@-1_&_o)=1.0 feature(m@1)=1.0 feature(o)=1.0  span[162..163]\nToken#136:m feature(,@1_&_f@2_&_o@3)=1.0 feature(e@-3_&_d@-2_&_o@-1)=1.0 feature(m_&_,@1_&_f@2)=1.0 feature(d@-2_&_o@-1_&_m)=1.0 feature(m_&_,@1)=1.0 feature(o@-1_&_m)=1.0 feature(,@1)=1.0 feature(m)=1.0  span[163..164]\nToken#137:, feature(f@1_&_o@2_&_r@3)=1.0 feature(d@-3_&_o@-2_&_m@-1)=1.0 feature(,_&_f@1_&_o@2)=1.0 feature(o@-2_&_m@-1_&_,)=1.0 feature(,_&_f@1)=1.0 feature(m@-1_&_,)=1.0 feature(f@1)=1.0 feature(,)=1.0  span[164..165]\nToken#138:f feature(o@1_&_r@2_&_t@3)=1.0 feature(o@-3_&_m@-2_&_,@-1)=1.0 feature(f_&_o@1_&_r@2)=1.0 feature(m@-2_&_,@-1_&_f)=1.0 feature(f_&_o@1)=1.0 feature(,@-1_&_f)=1.0 feature(o@1)=1.0 feature(f)=1.0  span[166..167]\nToken#139:o feature(r@1_&_t@2_&_h@3)=1.0 feature(m@-3_&_,@-2_&_f@-1)=1.0 feature(o_&_r@1_&_t@2)=1.0 feature(,@-2_&_f@-1_&_o)=1.0 feature(o_&_r@1)=1.0 feature(f@-1_&_o)=1.0 feature(r@1)=1.0 feature(o)=1.0  span[167..168]\nToken#140:r feature(t@1_&_h@2_&_e@3)=1.0 feature(,@-3_&_f@-2_&_o@-1)=1.0 feature(r_&_t@1_&_h@2)=1.0 feature(f@-2_&_o@-1_&_r)=1.0 feature(r_&_t@1)=1.0 feature(o@-1_&_r)=1.0 feature(t@1)=1.0 feature(r)=1.0  span[168..169]\nToken#141:t feature(h@1_&_e@2_&_u@3)=1.0 feature(f@-3_&_o@-2_&_r@-1)=1.0 feature(t_&_h@1_&_e@2)=1.0 feature(o@-2_&_r@-1_&_t)=1.0 feature(t_&_h@1)=1.0 feature(r@-1_&_t)=1.0 feature(h@1)=1.0 feature(t)=1.0  span[170..171]\nToken#142:h feature(e@1_&_u@2_&_s@3)=1.0 feature(o@-3_&_r@-2_&_t@-1)=1.0 feature(h_&_e@1_&_u@2)=1.0 feature(r@-2_&_t@-1_&_h)=1.0 feature(h_&_e@1)=1.0 feature(t@-1_&_h)=1.0 feature(e@1)=1.0 feature(h)=1.0  span[171..172]\nToken#143:e feature(u@1_&_s@2_&_e@3)=1.0 feature(r@-3_&_t@-2_&_h@-1)=1.0 feature(e_&_u@1_&_s@2)=1.0 feature(t@-2_&_h@-1_&_e)=1.0 feature(e_&_u@1)=1.0 feature(h@-1_&_e)=1.0 feature(u@1)=1.0 feature(e)=1.0  span[172..173]\nToken#144:u feature(s@1_&_e@2_&_r@3)=1.0 feature(t@-3_&_h@-2_&_e@-1)=1.0 feature(u_&_s@1_&_e@2)=1.0 feature(h@-2_&_e@-1_&_u)=1.0 feature(u_&_s@1)=1.0 feature(e@-1_&_u)=1.0 feature(s@1)=1.0 feature(u)=1.0  span[174..175]\nToken#145:s feature(e@1_&_r@2_&_s@3)=1.0 feature(h@-3_&_e@-2_&_u@-1)=1.0 feature(s_&_e@1_&_r@2)=1.0 feature(e@-2_&_u@-1_&_s)=1.0 feature(s_&_e@1)=1.0 feature(u@-1_&_s)=1.0 feature(e@1)=1.0 feature(s)=1.0  span[175..176]\nToken#146:e feature(r@1_&_s@2_&_o@3)=1.0 feature(e@-3_&_u@-2_&_s@-1)=1.0 feature(e_&_r@1_&_s@2)=1.0 feature(u@-2_&_s@-1_&_e)=1.0 feature(e_&_r@1)=1.0 feature(s@-1_&_e)=1.0 feature(r@1)=1.0 feature(e)=1.0  span[176..177]\nToken#147:r feature(s@1_&_o@2_&_f@3)=1.0 feature(u@-3_&_s@-2_&_e@-1)=1.0 feature(r_&_s@1_&_o@2)=1.0 feature(s@-2_&_e@-1_&_r)=1.0 feature(r_&_s@1)=1.0 feature(e@-1_&_r)=1.0 feature(s@1)=1.0 feature(r)=1.0  span[177..178]\nToken#148:s feature(o@1_&_f@2_&_t@3)=1.0 feature(s@-3_&_e@-2_&_r@-1)=1.0 feature(s_&_o@1_&_f@2)=1.0 feature(e@-2_&_r@-1_&_s)=1.0 feature(s_&_o@1)=1.0 feature(r@-1_&_s)=1.0 feature(o@1)=1.0 feature(s)=1.0  span[178..179]\nToken#149:o feature(f@1_&_t@2_&_h@3)=1.0 feature(e@-3_&_r@-2_&_s@-1)=1.0 feature(o_&_f@1_&_t@2)=1.0 feature(r@-2_&_s@-1_&_o)=1.0 feature(o_&_f@1)=1.0 feature(s@-1_&_o)=1.0 feature(f@1)=1.0 feature(o)=1.0  span[180..181]\nToken#150:f feature(t@1_&_h@2_&_e@3)=1.0 feature(r@-3_&_s@-2_&_o@-1)=1.0 feature(f_&_t@1_&_h@2)=1.0 feature(s@-2_&_o@-1_&_f)=1.0 feature(f_&_t@1)=1.0 feature(o@-1_&_f)=1.0 feature(t@1)=1.0 feature(f)=1.0  span[181..182]\nToken#151:t feature(h@1_&_e@2_&_s@3)=1.0 feature(s@-3_&_o@-2_&_f@-1)=1.0 feature(t_&_h@1_&_e@2)=1.0 feature(o@-2_&_f@-1_&_t)=1.0 feature(t_&_h@1)=1.0 feature(f@-1_&_t)=1.0 feature(h@1)=1.0 feature(t)=1.0  span[183..184]\nToken#152:h feature(e@1_&_s@2_&_o@3)=1.0 feature(o@-3_&_f@-2_&_t@-1)=1.0 feature(h_&_e@1_&_s@2)=1.0 feature(f@-2_&_t@-1_&_h)=1.0 feature(h_&_e@1)=1.0 feature(t@-1_&_h)=1.0 feature(e@1)=1.0 feature(h)=1.0  span[184..185]\nToken#153:e feature(s@1_&_o@2_&_f@3)=1.0 feature(f@-3_&_t@-2_&_h@-1)=1.0 feature(e_&_s@1_&_o@2)=1.0 feature(t@-2_&_h@-1_&_e)=1.0 feature(e_&_s@1)=1.0 feature(h@-1_&_e)=1.0 feature(s@1)=1.0 feature(e)=1.0  span[185..186]\nToken#154:s feature(o@1_&_f@2_&_t@3)=1.0 feature(t@-3_&_h@-2_&_e@-1)=1.0 feature(s_&_o@1_&_f@2)=1.0 feature(h@-2_&_e@-1_&_s)=1.0 feature(s_&_o@1)=1.0 feature(e@-1_&_s)=1.0 feature(o@1)=1.0 feature(s)=1.0  span[187..188]\nToken#155:o feature(f@1_&_t@2_&_w@3)=1.0 feature(h@-3_&_e@-2_&_s@-1)=1.0 feature(o_&_f@1_&_t@2)=1.0 feature(e@-2_&_s@-1_&_o)=1.0 feature(o_&_f@1)=1.0 feature(s@-1_&_o)=1.0 feature(f@1)=1.0 feature(o)=1.0  span[188..189]\nToken#156:f feature(t@1_&_w@2_&_a@3)=1.0 feature(e@-3_&_s@-2_&_o@-1)=1.0 feature(f_&_t@1_&_w@2)=1.0 feature(s@-2_&_o@-1_&_f)=1.0 feature(f_&_t@1)=1.0 feature(o@-1_&_f)=1.0 feature(t@1)=1.0 feature(f)=1.0  span[189..190]\nToken#157:t feature(w@1_&_a@2_&_r@3)=1.0 feature(s@-3_&_o@-2_&_f@-1)=1.0 feature(t_&_w@1_&_a@2)=1.0 feature(o@-2_&_f@-1_&_t)=1.0 feature(t_&_w@1)=1.0 feature(f@-1_&_t)=1.0 feature(w@1)=1.0 feature(t)=1.0  span[190..191]\nToken#158:w feature(a@1_&_r@2_&_e@3)=1.0 feature(o@-3_&_f@-2_&_t@-1)=1.0 feature(w_&_a@1_&_r@2)=1.0 feature(f@-2_&_t@-1_&_w)=1.0 feature(w_&_a@1)=1.0 feature(t@-1_&_w)=1.0 feature(a@1)=1.0 feature(w)=1.0  span[191..192]\nToken#159:a feature(r@1_&_e@2_&_.@3)=1.0 feature(f@-3_&_t@-2_&_w@-1)=1.0 feature(a_&_r@1_&_e@2)=1.0 feature(t@-2_&_w@-1_&_a)=1.0 feature(a_&_r@1)=1.0 feature(w@-1_&_a)=1.0 feature(r@1)=1.0 feature(a)=1.0  span[192..193]\nToken#160:r feature(e@1_&_.@2_&_<END0>@3)=1.0 feature(t@-3_&_w@-2_&_a@-1)=1.0 feature(r_&_e@1_&_.@2)=1.0 feature(w@-2_&_a@-1_&_r)=1.0 feature(r_&_e@1)=1.0 feature(a@-1_&_r)=1.0 feature(e@1)=1.0 feature(r)=1.0  span[193..194]\nToken#161:e feature(.@1_&_<END0>@2_&_<END1>@3)=1.0 feature(w@-3_&_a@-2_&_r@-1)=1.0 feature(e_&_.@1_&_<END0>@2)=1.0 feature(a@-2_&_r@-1_&_e)=1.0 feature(e_&_.@1)=1.0 feature(r@-1_&_e)=1.0 feature(.@1)=1.0 feature(e)=1.0  span[194..195]\nToken#162:. feature(<END0>@1_&_<END1>@2_&_<END2>@3)=1.0 feature(a@-3_&_r@-2_&_e@-1)=1.0 feature(._&_<END0>@1_&_<END1>@2)=1.0 feature(r@-2_&_e@-1_&_.)=1.0 feature(._&_<END0>@1)=1.0 feature(e@-1_&_.)=1.0 feature(<END0>@1)=1.0 feature(.)=1.0  span[195..196]\n\nname: array:1\ntarget: 0: start (0)\n1: notstart (1)\n2: notstart (1)\n3: start (0)\n4: notstart (1)\n5: notstart (1)\n6: notstart (1)\n7: notstart (1)\n8: notstart (1)\n9: notstart (1)\n10: start (0)\n11: notstart (1)\n12: start (0)\n13: notstart (1)\n14: notstart (1)\n15: start (0)\n16: notstart (1)\n17: notstart (1)\n18: start (0)\n19: notstart (1)\n20: notstart (1)\n21: notstart (1)\n22: notstart (1)\n23: notstart (1)\n24: notstart (1)\n25: notstart (1)\n26: start (0)\n27: notstart (1)\n28: notstart (1)\n29: start (0)\n30: notstart (1)\n31: notstart (1)\n32: start (0)\n33: notstart (1)\n34: notstart (1)\n35: notstart (1)\n36: notstart (1)\n37: notstart (1)\n38: notstart (1)\n39: notstart (1)\n\ninput: Token#0:t feature(h@1_&_e@2_&_f@3)=1.0 feature(<START2>@-3_&_<START1>@-2_&_<START0>@-1)=1.0 feature(t_&_h@1_&_e@2)=1.0 feature(<START1>@-2_&_<START0>@-1_&_t)=1.0 feature(t_&_h@1)=1.0 feature(<START0>@-1_&_t)=1.0 feature(h@1)=1.0 feature(t)=1.0  span[0..1]\nToken#1:h feature(e@1_&_f@2_&_r@3)=1.0 feature(<START1>@-3_&_<START0>@-2_&_t@-1)=1.0 feature(h_&_e@1_&_f@2)=1.0 feature(<START0>@-2_&_t@-1_&_h)=1.0 feature(h_&_e@1)=1.0 feature(t@-1_&_h)=1.0 feature(e@1)=1.0 feature(h)=1.0  span[1..2]\nToken#2:e feature(f@1_&_r@2_&_e@3)=1.0 feature(<START0>@-3_&_t@-2_&_h@-1)=1.0 feature(e_&_f@1_&_r@2)=1.0 feature(t@-2_&_h@-1_&_e)=1.0 feature(e_&_f@1)=1.0 feature(h@-1_&_e)=1.0 feature(f@1)=1.0 feature(e)=1.0  span[2..3]\nToken#3:f feature(r@1_&_e@2_&_e@3)=1.0 feature(t@-3_&_h@-2_&_e@-1)=1.0 feature(f_&_r@1_&_e@2)=1.0 feature(h@-2_&_e@-1_&_f)=1.0 feature(f_&_r@1)=1.0 feature(e@-1_&_f)=1.0 feature(r@1)=1.0 feature(f)=1.0  span[4..5]\nToken#4:r feature(e@1_&_e@2_&_d@3)=1.0 feature(h@-3_&_e@-2_&_f@-1)=1.0 feature(r_&_e@1_&_e@2)=1.0 feature(e@-2_&_f@-1_&_r)=1.0 feature(r_&_e@1)=1.0 feature(f@-1_&_r)=1.0 feature(e@1)=1.0 feature(r)=1.0  span[5..6]\nToken#5:e feature(e@1_&_d@2_&_o@3)=1.0 feature(e@-3_&_f@-2_&_r@-1)=1.0 feature(e_&_e@1_&_d@2)=1.0 feature(f@-2_&_r@-1_&_e)=1.0 feature(e_&_e@1)=1.0 feature(r@-1_&_e)=1.0 feature(e@1)=1.0 feature(e)=1.0  span[6..7]\nToken#6:e feature(d@1_&_o@2_&_m@3)=1.0 feature(f@-3_&_r@-2_&_e@-1)=1.0 feature(e_&_d@1_&_o@2)=1.0 feature(r@-2_&_e@-1_&_e)=1.0 feature(e_&_d@1)=1.0 feature(e@-1_&_e)=1.0 feature(d@1)=1.0 feature(e)=1.0  span[7..8]\nToken#7:d feature(o@1_&_m@2_&_t@3)=1.0 feature(r@-3_&_e@-2_&_e@-1)=1.0 feature(d_&_o@1_&_m@2)=1.0 feature(e@-2_&_e@-1_&_d)=1.0 feature(d_&_o@1)=1.0 feature(e@-1_&_d)=1.0 feature(o@1)=1.0 feature(d)=1.0  span[8..9]\nToken#8:o feature(m@1_&_t@2_&_o@3)=1.0 feature(e@-3_&_e@-2_&_d@-1)=1.0 feature(o_&_m@1_&_t@2)=1.0 feature(e@-2_&_d@-1_&_o)=1.0 feature(o_&_m@1)=1.0 feature(d@-1_&_o)=1.0 feature(m@1)=1.0 feature(o)=1.0  span[9..10]\nToken#9:m feature(t@1_&_o@2_&_r@3)=1.0 feature(e@-3_&_d@-2_&_o@-1)=1.0 feature(m_&_t@1_&_o@2)=1.0 feature(d@-2_&_o@-1_&_m)=1.0 feature(m_&_t@1)=1.0 feature(o@-1_&_m)=1.0 feature(t@1)=1.0 feature(m)=1.0  span[10..11]\nToken#10:t feature(o@1_&_r@2_&_u@3)=1.0 feature(d@-3_&_o@-2_&_m@-1)=1.0 feature(t_&_o@1_&_r@2)=1.0 feature(o@-2_&_m@-1_&_t)=1.0 feature(t_&_o@1)=1.0 feature(m@-1_&_t)=1.0 feature(o@1)=1.0 feature(t)=1.0  span[12..13]\nToken#11:o feature(r@1_&_u@2_&_n@3)=1.0 feature(o@-3_&_m@-2_&_t@-1)=1.0 feature(o_&_r@1_&_u@2)=1.0 feature(m@-2_&_t@-1_&_o)=1.0 feature(o_&_r@1)=1.0 feature(t@-1_&_o)=1.0 feature(r@1)=1.0 feature(o)=1.0  span[13..14]\nToken#12:r feature(u@1_&_n@2_&_t@3)=1.0 feature(m@-3_&_t@-2_&_o@-1)=1.0 feature(r_&_u@1_&_n@2)=1.0 feature(t@-2_&_o@-1_&_r)=1.0 feature(r_&_u@1)=1.0 feature(o@-1_&_r)=1.0 feature(u@1)=1.0 feature(r)=1.0  span[15..16]\nToken#13:u feature(n@1_&_t@2_&_h@3)=1.0 feature(t@-3_&_o@-2_&_r@-1)=1.0 feature(u_&_n@1_&_t@2)=1.0 feature(o@-2_&_r@-1_&_u)=1.0 feature(u_&_n@1)=1.0 feature(r@-1_&_u)=1.0 feature(n@1)=1.0 feature(u)=1.0  span[16..17]\nToken#14:n feature(t@1_&_h@2_&_e@3)=1.0 feature(o@-3_&_r@-2_&_u@-1)=1.0 feature(n_&_t@1_&_h@2)=1.0 feature(r@-2_&_u@-1_&_n)=1.0 feature(n_&_t@1)=1.0 feature(u@-1_&_n)=1.0 feature(t@1)=1.0 feature(n)=1.0  span[17..18]\nToken#15:t feature(h@1_&_e@2_&_p@3)=1.0 feature(r@-3_&_u@-2_&_n@-1)=1.0 feature(t_&_h@1_&_e@2)=1.0 feature(u@-2_&_n@-1_&_t)=1.0 feature(t_&_h@1)=1.0 feature(n@-1_&_t)=1.0 feature(h@1)=1.0 feature(t)=1.0  span[19..20]\nToken#16:h feature(e@1_&_p@2_&_r@3)=1.0 feature(u@-3_&_n@-2_&_t@-1)=1.0 feature(h_&_e@1_&_p@2)=1.0 feature(n@-2_&_t@-1_&_h)=1.0 feature(h_&_e@1)=1.0 feature(t@-1_&_h)=1.0 feature(e@1)=1.0 feature(h)=1.0  span[20..21]\nToken#17:e feature(p@1_&_r@2_&_o@3)=1.0 feature(n@-3_&_t@-2_&_h@-1)=1.0 feature(e_&_p@1_&_r@2)=1.0 feature(t@-2_&_h@-1_&_e)=1.0 feature(e_&_p@1)=1.0 feature(h@-1_&_e)=1.0 feature(p@1)=1.0 feature(e)=1.0  span[21..22]\nToken#18:p feature(r@1_&_o@2_&_g@3)=1.0 feature(t@-3_&_h@-2_&_e@-1)=1.0 feature(p_&_r@1_&_o@2)=1.0 feature(h@-2_&_e@-1_&_p)=1.0 feature(p_&_r@1)=1.0 feature(e@-1_&_p)=1.0 feature(r@1)=1.0 feature(p)=1.0  span[23..24]\nToken#19:r feature(o@1_&_g@2_&_r@3)=1.0 feature(h@-3_&_e@-2_&_p@-1)=1.0 feature(r_&_o@1_&_g@2)=1.0 feature(e@-2_&_p@-1_&_r)=1.0 feature(r_&_o@1)=1.0 feature(p@-1_&_r)=1.0 feature(o@1)=1.0 feature(r)=1.0  span[24..25]\nToken#20:o feature(g@1_&_r@2_&_a@3)=1.0 feature(e@-3_&_p@-2_&_r@-1)=1.0 feature(o_&_g@1_&_r@2)=1.0 feature(p@-2_&_r@-1_&_o)=1.0 feature(o_&_g@1)=1.0 feature(r@-1_&_o)=1.0 feature(g@1)=1.0 feature(o)=1.0  span[25..26]\nToken#21:g feature(r@1_&_a@2_&_m@3)=1.0 feature(p@-3_&_r@-2_&_o@-1)=1.0 feature(g_&_r@1_&_a@2)=1.0 feature(r@-2_&_o@-1_&_g)=1.0 feature(g_&_r@1)=1.0 feature(o@-1_&_g)=1.0 feature(r@1)=1.0 feature(g)=1.0  span[26..27]\nToken#22:r feature(a@1_&_m@2_&_,@3)=1.0 feature(r@-3_&_o@-2_&_g@-1)=1.0 feature(r_&_a@1_&_m@2)=1.0 feature(o@-2_&_g@-1_&_r)=1.0 feature(r_&_a@1)=1.0 feature(g@-1_&_r)=1.0 feature(a@1)=1.0 feature(r)=1.0  span[27..28]\nToken#23:a feature(m@1_&_,@2_&_f@3)=1.0 feature(o@-3_&_g@-2_&_r@-1)=1.0 feature(a_&_m@1_&_,@2)=1.0 feature(g@-2_&_r@-1_&_a)=1.0 feature(a_&_m@1)=1.0 feature(r@-1_&_a)=1.0 feature(m@1)=1.0 feature(a)=1.0  span[28..29]\nToken#24:m feature(,@1_&_f@2_&_o@3)=1.0 feature(g@-3_&_r@-2_&_a@-1)=1.0 feature(m_&_,@1_&_f@2)=1.0 feature(r@-2_&_a@-1_&_m)=1.0 feature(m_&_,@1)=1.0 feature(a@-1_&_m)=1.0 feature(,@1)=1.0 feature(m)=1.0  span[29..30]\nToken#25:, feature(f@1_&_o@2_&_r@3)=1.0 feature(r@-3_&_a@-2_&_m@-1)=1.0 feature(,_&_f@1_&_o@2)=1.0 feature(a@-2_&_m@-1_&_,)=1.0 feature(,_&_f@1)=1.0 feature(m@-1_&_,)=1.0 feature(f@1)=1.0 feature(,)=1.0  span[30..31]\nToken#26:f feature(o@1_&_r@2_&_a@3)=1.0 feature(a@-3_&_m@-2_&_,@-1)=1.0 feature(f_&_o@1_&_r@2)=1.0 feature(m@-2_&_,@-1_&_f)=1.0 feature(f_&_o@1)=1.0 feature(,@-1_&_f)=1.0 feature(o@1)=1.0 feature(f)=1.0  span[32..33]\nToken#27:o feature(r@1_&_a@2_&_n@3)=1.0 feature(m@-3_&_,@-2_&_f@-1)=1.0 feature(o_&_r@1_&_a@2)=1.0 feature(,@-2_&_f@-1_&_o)=1.0 feature(o_&_r@1)=1.0 feature(f@-1_&_o)=1.0 feature(r@1)=1.0 feature(o)=1.0  span[33..34]\nToken#28:r feature(a@1_&_n@2_&_y@3)=1.0 feature(,@-3_&_f@-2_&_o@-1)=1.0 feature(r_&_a@1_&_n@2)=1.0 feature(f@-2_&_o@-1_&_r)=1.0 feature(r_&_a@1)=1.0 feature(o@-1_&_r)=1.0 feature(a@1)=1.0 feature(r)=1.0  span[34..35]\nToken#29:a feature(n@1_&_y@2_&_p@3)=1.0 feature(f@-3_&_o@-2_&_r@-1)=1.0 feature(a_&_n@1_&_y@2)=1.0 feature(o@-2_&_r@-1_&_a)=1.0 feature(a_&_n@1)=1.0 feature(r@-1_&_a)=1.0 feature(n@1)=1.0 feature(a)=1.0  span[36..37]\nToken#30:n feature(y@1_&_p@2_&_u@3)=1.0 feature(o@-3_&_r@-2_&_a@-1)=1.0 feature(n_&_y@1_&_p@2)=1.0 feature(r@-2_&_a@-1_&_n)=1.0 feature(n_&_y@1)=1.0 feature(a@-1_&_n)=1.0 feature(y@1)=1.0 feature(n)=1.0  span[37..38]\nToken#31:y feature(p@1_&_u@2_&_r@3)=1.0 feature(r@-3_&_a@-2_&_n@-1)=1.0 feature(y_&_p@1_&_u@2)=1.0 feature(a@-2_&_n@-1_&_y)=1.0 feature(y_&_p@1)=1.0 feature(n@-1_&_y)=1.0 feature(p@1)=1.0 feature(y)=1.0  span[38..39]\nToken#32:p feature(u@1_&_r@2_&_p@3)=1.0 feature(a@-3_&_n@-2_&_y@-1)=1.0 feature(p_&_u@1_&_r@2)=1.0 feature(n@-2_&_y@-1_&_p)=1.0 feature(p_&_u@1)=1.0 feature(y@-1_&_p)=1.0 feature(u@1)=1.0 feature(p)=1.0  span[40..41]\nToken#33:u feature(r@1_&_p@2_&_o@3)=1.0 feature(n@-3_&_y@-2_&_p@-1)=1.0 feature(u_&_r@1_&_p@2)=1.0 feature(y@-2_&_p@-1_&_u)=1.0 feature(u_&_r@1)=1.0 feature(p@-1_&_u)=1.0 feature(r@1)=1.0 feature(u)=1.0  span[41..42]\nToken#34:r feature(p@1_&_o@2_&_s@3)=1.0 feature(y@-3_&_p@-2_&_u@-1)=1.0 feature(r_&_p@1_&_o@2)=1.0 feature(p@-2_&_u@-1_&_r)=1.0 feature(r_&_p@1)=1.0 feature(u@-1_&_r)=1.0 feature(p@1)=1.0 feature(r)=1.0  span[42..43]\nToken#35:p feature(o@1_&_s@2_&_e@3)=1.0 feature(p@-3_&_u@-2_&_r@-1)=1.0 feature(p_&_o@1_&_s@2)=1.0 feature(u@-2_&_r@-1_&_p)=1.0 feature(p_&_o@1)=1.0 feature(r@-1_&_p)=1.0 feature(o@1)=1.0 feature(p)=1.0  span[43..44]\nToken#36:o feature(s@1_&_e@2_&_.@3)=1.0 feature(u@-3_&_r@-2_&_p@-1)=1.0 feature(o_&_s@1_&_e@2)=1.0 feature(r@-2_&_p@-1_&_o)=1.0 feature(o_&_s@1)=1.0 feature(p@-1_&_o)=1.0 feature(s@1)=1.0 feature(o)=1.0  span[44..45]\nToken#37:s feature(e@1_&_.@2_&_<END0>@3)=1.0 feature(r@-3_&_p@-2_&_o@-1)=1.0 feature(s_&_e@1_&_.@2)=1.0 feature(p@-2_&_o@-1_&_s)=1.0 feature(s_&_e@1)=1.0 feature(o@-1_&_s)=1.0 feature(e@1)=1.0 feature(s)=1.0  span[45..46]\nToken#38:e feature(.@1_&_<END0>@2_&_<END1>@3)=1.0 feature(p@-3_&_o@-2_&_s@-1)=1.0 feature(e_&_.@1_&_<END0>@2)=1.0 feature(o@-2_&_s@-1_&_e)=1.0 feature(e_&_.@1)=1.0 feature(s@-1_&_e)=1.0 feature(.@1)=1.0 feature(e)=1.0  span[46..47]\nToken#39:. feature(<END0>@1_&_<END1>@2_&_<END2>@3)=1.0 feature(o@-3_&_s@-2_&_e@-1)=1.0 feature(._&_<END0>@1_&_<END1>@2)=1.0 feature(s@-2_&_e@-1_&_.)=1.0 feature(._&_<END0>@1)=1.0 feature(e@-1_&_.)=1.0 feature(<END0>@1)=1.0 feature(.)=1.0  span[47..48]\n\n";

}// TestSpacePipe
